__all__ = ["DomainAuthConfig"]

from django.apps import AppConfig


class DomainAuthConfig(AppConfig):
    name = 'spider_domainauth'
    label = 'spider_domainauth'
    verbose_name = 'db based domain authentication'
    spider_url_path = 'domainauth/'



__all__ = ["ReverseTokenManager", "BaseReverseToken"]

from django.db import models
from django.conf import settings
from django.urls import reverse

from spkcspider.apps.spider.constants import (
    hex_size_of_bigid
)
from spkcspider.apps.spider.helpers import create_b64_id_token

MAX_TOKEN_B64_SIZE = 90
_striptoken = getattr(settings, "TOKEN_SIZE", 30)*4//3


class ReverseTokenManager(models.Manager):

    def create(self, *, secret=None, token=None, **kwargs):
        ret = self.get_queryset().create(**kwargs)
        ret.initialize_token(secret or token)
        ret.save(update_fields=["token"])
        return ret


class BaseReverseToken(models.Model):
    token = models.CharField(
        max_length=MAX_TOKEN_B64_SIZE+hex_size_of_bigid*2+4, null=True,
        blank=True, unique=True
    )

    objects = ReverseTokenManager()

    class Meta:
        abstract = True

    def __str__(self):
        if self.token:
            return "{}...".format(self.token[:-_striptoken])
        else:
            return "<empty>"

    def get_absolute_url(self):
        return reverse("spider_domainauth:db-domainauth")

    def initialize_token(self, secret=None):
        if secret:
            self.token = "{}_{}".format(self.id, secret)
        else:
            self.token = create_b64_id_token(
                self.id, "_", getattr(settings, "TOKEN_SIZE", 30)
            )

    def _get_secret(self):
        if not self.token:
            return None
        return self.token.split("_", 1)[1]

    secret = property(fget=_get_secret, fset=initialize_token)



# Generated by Django 2.2.1 on 2019-05-30 10:08

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('spider_base', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ReverseToken',
            fields=[
                ('id', models.BigAutoField(editable=False, primary_key=True, serialize=False)),
                ('token', models.CharField(blank=True, max_length=126, null=True, unique=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('assignedcontent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='spider_base.AssignedContent')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['spider_domainauth', 'spider_domainauth.migrations']

package_data = \
{'': ['*']}

install_requires = \
['django>=2.0']

setup_kwargs = {
    'name': 'spkcspider-domainauth',
    'version': '0.3',
    'description': 'Helper for spkcspiders domain authentication',
    'long_description': '\n\nHelper for db based domain auth\n\n# Installation\n\n~~~~ sh\npip install spkcspider-domainauth\n~~~~\n\nsettings:\n\n~~~~\n...\nINSTALLED_APPS = [\n...\n    spider_domainauth\n...\n]\n~~~~\n\n# Usage:\n\n~~~~ python\nfrom spider_domainauth.models import ReverseToken\n\n# overloaded method\nReverseToken.object.create(secret="kskls")\n~~~~\n\n\n# TODO:\n* overload other manager methods\n* better examples\n',
    'author': 'Alexander Kaftan',
    'author_email': None,
    'url': 'https://github.com/spkcspider/spkcspider-domainauth',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5',
}


setup(**setup_kwargs)



__all__ = ("ReverseTokenView", )

import logging
from importlib import import_module

from django.conf import settings
from django.apps import apps
from django.shortcuts import get_object_or_404
from django.http import Http404, HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.utils.decorators import method_decorator
from django.views import View


logger = logging.getLogger(__name__)


def stubhttp404_handler(*args, **kwargs):
    pass


_http404_handler = stubhttp404_handler

if "spkcspider.apps.spider" in settings.INSTALLED_APPS:
    _http404_handler = "spkcspider.apps.spider.functions.rate_limit_default"

if "spkcspider.apps.spider.apps.SpiderBaseConfig" in settings.INSTALLED_APPS:
    _http404_handler = "spkcspider.apps.spider.functions.rate_limit_default"

if getattr(settings, "SPIDER_RATELIMIT_FUNC", None):
    _http404_handler = settings.SPIDER_RATELIMIT_FUNC

if isinstance(_http404_handler, str):
    module, name = _http404_handler.rsplit(".", 1)
    _http404_handler = getattr(import_module(module), name)


class ReverseTokenView(View):
    model = apps.get_model(
        *getattr(
            settings,
            "DOMAIN_AUTH_MODEL",
            "spider_domainauth.ReverseToken"
        ).split(".", 1)
    )
    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        try:
            return super().dispatch(request, *args, **kwargs)
        except Http404:
            return _http404_handler(self, request)

    def post(self, request, *args, **kwargs):
        payload = self.request.POST.get(
            "payload"
        )
        newtoken = self.request.POST.get("token", None)
        if not payload or not newtoken:
            raise Http404()

        self.object = get_object_or_404(
            self.model, token=payload
        )
        self.object.token = "{}_{}".format(self.object.id, newtoken)
        self.object.save(update_fields=["token"])
        return HttpResponse(status=200)

    def options(self, request, *args, **kwargs):
        ret = super().options()
        ret["Access-Control-Allow-Origin"] = "*"
        ret["Access-Control-Allow-Methods"] = "POST, OPTIONS"
        return ret



__all__ = ["ReverseToken"]

from django.db import models

from .abstract_models import BaseReverseToken


class ReverseToken(BaseReverseToken):
    id = models.BigAutoField(primary_key=True, editable=False)
    created = models.DateTimeField(auto_now_add=True, editable=False)

    # for automatic deletion
    assignedcontent = models.ForeignKey(
        "spider_base.AssignedContent", on_delete=models.CASCADE,
        related_name="+", null=True, blank=True
    )



from django.urls import path

from .views import ReverseTokenView

app_name = "spider_domainauth"


urlpatterns = [
    path(
        'db/',
        ReverseTokenView.as_view(),
        name='db-domainauth'
    )
]
